/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef DOCUMENTCONTROL_H_INCLUDED
#define DOCUMENTCONTROL_H_INCLUDED

#include "MainMenu.h"
#include "FileHolder.h"
#include "FindDialog.h"
#include "GotoDialog.h"

#include <gtkmm.h>

#include <c4scripter/c4s-documentwindow.h>
#include <c4scripter/c4s-document.h>
#include <c4scripter/c4s-preferencesdialog.h>
#include <c4scripter/c4s-preferences.h>

#include <memory>

class DocumentControl
{
public:
  DocumentControl(MainMenu& menu, FileHolder& file_holder, C4sPreferences* preferences, Gtk::Window& parent);
  ~DocumentControl();

private:
  void on_document_added(C4sDocumentWindow* window);
  void on_document_changed(C4sDocumentWindow* window);

  void on_undo();
  void on_redo();

  void on_cut();
  void on_copy();
  void on_paste();

  void on_find();
  void on_replace();
  void on_goto();
  void on_preferences();

  static void on_can_undo_static(GObject* object, GParamSpec* pspec, gpointer info);
  static void on_can_redo_static(GObject* object, GParamSpec* pspec, gpointer info);
  
  void on_preferences_destroy();
  static void on_preferences_destroy_static(GtkWidget* widget, gpointer user_data) { static_cast<DocumentControl*>(user_data)->on_preferences_destroy(); }

  MainMenu& m_menu;
  FileHolder& m_file_holder;
  C4sPreferences* m_preferences;
  Gtk::Window& m_parent;

  std::auto_ptr<FindDialog> m_find_dialog;
  std::auto_ptr<GotoDialog> m_goto_dialog;
  //std::auto_ptr<PreferencesDialog> m_preferences_dialog;
  C4sPreferencesDialog* m_preferences_dialog;
};

#endif // DOCUMENTCONTROL_H_INCLUDED
