/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_CUSTOMFUNCTION_H__
#define __C4S_CUSTOMFUNCTION_H__

#include <c4scripter/c4s-function.h>
#include <c4scripter/c4s-highlighter.h>

#include <gtk/gtktextiter.h>

G_BEGIN_DECLS

#define C4S_TYPE_CUSTOM_FUNCTION_ACCESS			(c4s_custom_function_access_get_type())

#define C4S_TYPE_CUSTOM_FUNCTION			(c4s_custom_function_get_type())
#define C4S_CUSTOM_FUNCTION(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_CUSTOM_FUNCTION, C4sCustomFunction))
#define C4S_CUSTOM_FUNCTION_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_CUSTOM_FUNCTION, C4sCustomFunctionClass))
#define C4S_IS_CUSTOM_FUNCTION(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_CUSTOM_FUNCTION))
#define C4S_IS_CUSTOM_FUNCTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_CUSTOM_FUNCTION))
#define C4S_CUSTOM_FUNCTION_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_CUSTOM_FUNCTION, C4sCustomFunctionClass))

typedef struct _C4sCustomFunctionClass C4sCustomFunctionClass;
typedef struct _C4sCustomFunction C4sCustomFunction;

typedef enum _C4sCustomFunctionAccess {
  C4S_CUSTOM_FUNCTION_ACCESS_PRIVATE,
  C4S_CUSTOM_FUNCTION_ACCESS_PROTECTED,
  C4S_CUSTOM_FUNCTION_ACCESS_PUBLIC,
  C4S_CUSTOM_FUNCTION_ACCESS_GLOBAL
} C4sCustomFunctionAccess;

struct _C4sCustomFunctionClass
{
  C4sFunctionClass parent_class;

  void(*access_changed)(
    C4sCustomFunction*,
    C4sCustomFunctionAccess
  );

  void(*name_changed)(
    C4sCustomFunction*,
    const gchar*
  );
};

struct _C4sCustomFunction
{
  C4sFunction parent_instance;
};

GType
c4s_custom_function_access_get_type(void) G_GNUC_CONST;

GType
c4s_custom_function_get_type(void) G_GNUC_CONST;

C4sCustomFunction*
c4s_custom_function_new(GtkTextIter* pos,
                        const C4sHighlighter* highlighter);

gboolean
c4s_custom_function_update(C4sCustomFunction* function,
                           const C4sHighlighter* highlighter);

C4sCustomFunctionAccess
c4s_custom_function_get_access(const C4sCustomFunction* function);

void
c4s_custom_function_get_begin(const C4sCustomFunction* function,
                              GtkTextIter* iter);

void
c4s_custom_function_get_end(const C4sCustomFunction* function,
                            GtkTextIter* iter);

void
c4s_custom_function_get_access_begin(const C4sCustomFunction* function,
                                     GtkTextIter* iter);

gint
c4s_custom_function_compare(const C4sCustomFunction* first,
                            const C4sCustomFunction* second);

G_END_DECLS

#endif /* __C4S_CUSTOMFUNCTION_H__ */
