/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-optiongroup.h>

#include <gtk/gtklabel.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkalignment.h>

static GtkFrameClass* parent_class = NULL;

static void
c4s_option_group_init(C4sOptionGroup* group)
{
  GtkWidget* alignment;

  group->vbox = gtk_vbox_new(FALSE, 4);
  gtk_container_set_border_width(GTK_CONTAINER(group->vbox), 8);
  gtk_widget_show(group->vbox);

  alignment = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
  gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 12, 0);
  gtk_container_add(GTK_CONTAINER(alignment), group->vbox);
  gtk_widget_show(alignment);

  gtk_frame_set_shadow_type(GTK_FRAME(group), GTK_SHADOW_NONE);
  gtk_container_add(GTK_CONTAINER(group), alignment);
}

static void
c4s_option_group_class_init(C4sOptionGroupClass* klass)
{
  parent_class = GTK_FRAME_CLASS(g_type_class_peek_parent(klass));
}

GType
c4s_option_group_get_type(void)
{
  static GType our_type = 0;

  if(our_type == 0)
  {
    static const GTypeInfo our_info = {
      sizeof(C4sOptionGroupClass),
      (GBaseInitFunc)NULL,
      (GBaseFinalizeFunc)NULL,
      (GClassInitFunc)c4s_option_group_class_init,
      NULL,
      NULL,
      sizeof(C4sOptionGroup),
      0,
      (GInstanceInitFunc)c4s_option_group_init,
      NULL
    };

    our_type = g_type_register_static(
      GTK_TYPE_FRAME,
      "C4sOptionGroup",
      &our_info,
      0
    );
  }

  return our_type;
}

C4sOptionGroup*
c4s_option_group_new(const gchar* label)
{
  GObject* object;
  gchar* markup;

  markup = g_markup_printf_escaped("<b>%s</b>", label);
  object = g_object_new(C4S_TYPE_OPTION_GROUP, "label", markup, NULL);
  gtk_label_set_use_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(object))), TRUE);
  g_free(markup);

  return C4S_OPTION_GROUP(object);
}
