/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-preferencesfonts.h>

#include <gtk/gtkfontbutton.h>

#include <string.h>

typedef struct _C4sPreferencesFontsPrivate C4sPreferencesFontsPrivate;
struct _C4sPreferencesFontsPrivate
{
  C4sPreferences* preferences;

  GtkWidget* font_button;
};

#define C4S_PREFERENCES_FONTS_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE( \
    (o), \
    C4S_TYPE_PREFERENCES_FONTS, \
    C4sPreferencesFontsPrivate \
  ))

enum
{
  PROP_0,

  PROP_PREFERENCES
};

static C4sPreferencesPageClass* parent_class = NULL;

static void
c4s_preferences_fonts_notify_cb(GObject* object,
                                GParamSpec* pspec,
                                gpointer user_data);

static void
c4s_preferences_fonts_font_set_cb(G_GNUC_UNUSED GtkFontButton* widget,
                                  gpointer user_data)
{
  C4sPreferencesFonts* fonts;
  C4sPreferencesFontsPrivate* priv;
  PangoFontDescription* font;

  fonts = C4S_PREFERENCES_FONTS(user_data);
  priv = C4S_PREFERENCES_FONTS_PRIVATE(fonts);
  
  if(priv->preferences != NULL)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_fonts_notify_cb),
      fonts
    );

    font = pango_font_description_from_string(
      gtk_font_button_get_font_name(GTK_FONT_BUTTON(priv->font_button))
    );

    c4s_preferences_set_font(priv->preferences, font);
    pango_font_description_free(font);

    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_fonts_notify_cb),
      fonts
    );
  }
}

static void
c4s_preferences_fonts_notify_cb(GObject* object,
                                GParamSpec* pspec,
                                gpointer user_data)
{
  gchar* fontname;

  C4sPreferencesFonts* fonts;
  C4sPreferencesFontsPrivate* priv;

  fonts = C4S_PREFERENCES_FONTS(user_data);
  priv = C4S_PREFERENCES_FONTS_PRIVATE(fonts);

  g_signal_handlers_block_by_func(
    G_OBJECT(priv->font_button),
    G_CALLBACK(c4s_preferences_fonts_font_set_cb),
    fonts
  );

  if(strcmp(pspec->name, "font") == 0)
  {
    fontname = pango_font_description_to_string(
      c4s_preferences_get_font(priv->preferences)
    );

    gtk_font_button_set_font_name(
      GTK_FONT_BUTTON(priv->font_button),
      fontname
    );
    
    g_free(fontname);
  }

  g_signal_handlers_unblock_by_func(
    G_OBJECT(priv->font_button),
    G_CALLBACK(c4s_preferences_fonts_font_set_cb),
    fonts
  );
}

static void
c4s_preferences_fonts_init(C4sPreferencesFonts* fonts)
{
  C4sPreferencesFontsPrivate* priv;
  C4sOptionGroup* group;

  priv = C4S_PREFERENCES_FONTS_PRIVATE(fonts);

  priv->preferences = NULL;

  priv->font_button = gtk_font_button_new();
  gtk_widget_show(priv->font_button);

  group = c4s_preferences_page_add_option_group(
    C4S_PREFERENCES_PAGE(fonts),
    "Text"
  );

  gtk_box_pack_start(
    GTK_BOX(group->vbox),
    priv->font_button,
    FALSE,
    FALSE,
    0
  );
  
  g_signal_connect_after(
    G_OBJECT(priv->font_button),
    "font-set",
    G_CALLBACK(c4s_preferences_fonts_font_set_cb),
    fonts
  );

  gtk_widget_set_sensitive(GTK_WIDGET(fonts), FALSE);
}

static void
c4s_preferences_fonts_set_preferences(C4sPreferencesFonts* fonts,
                                       C4sPreferences* preferences)
{
  C4sPreferencesFontsPrivate* priv;
  gchar* fontname;

  priv = C4S_PREFERENCES_FONTS_PRIVATE(fonts);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_disconnect_by_func(
      G_OBJECT(preferences),
      G_CALLBACK(c4s_preferences_fonts_notify_cb),
      fonts
    );

    g_object_unref(G_OBJECT(preferences));

    if(preferences == NULL)
      gtk_widget_set_sensitive(GTK_WIDGET(fonts), FALSE);
  }

  if(preferences != NULL)
  {
    g_signal_connect(
      G_OBJECT(preferences),
      "notify",
      G_CALLBACK(c4s_preferences_fonts_notify_cb),
      fonts
    );

    g_signal_handlers_block_by_func(
      G_OBJECT(priv->font_button),
      G_CALLBACK(c4s_preferences_fonts_font_set_cb),
      fonts
    );

    fontname = pango_font_description_to_string(
      c4s_preferences_get_font(preferences)
    );

    gtk_font_button_set_font_name(
      GTK_FONT_BUTTON(priv->font_button),
      fontname
    );

    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->font_button),
      G_CALLBACK(c4s_preferences_fonts_font_set_cb),
      fonts
    );

    g_object_ref(G_OBJECT(preferences));

    if(priv->preferences == NULL)
      gtk_widget_set_sensitive(GTK_WIDGET(fonts), TRUE);
  }

  priv->preferences = preferences;
}

static void
c4s_preferences_fonts_finalize(GObject* object)
{
  C4sPreferencesFonts* fonts;
  C4sPreferencesFontsPrivate* priv;
  
  fonts = C4S_PREFERENCES_FONTS(object);
  priv = C4S_PREFERENCES_FONTS_PRIVATE(fonts);

  if(priv->preferences != NULL)
    g_object_unref(priv->preferences);

  g_signal_handlers_disconnect_by_func(
    G_OBJECT(priv->preferences),
    G_CALLBACK(c4s_preferences_fonts_notify_cb),
    fonts
  );

  G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
c4s_preferences_fonts_set_property(GObject* object,
                                   guint prop_id,
                                   const GValue* value,
                                   GParamSpec* pspec)
{
  C4sPreferencesFonts* fonts;
  C4sPreferencesFontsPrivate* priv;

  g_return_if_fail(C4S_IS_PREFERENCES_FONTS(object));

  fonts = C4S_PREFERENCES_FONTS(object);
  priv = C4S_PREFERENCES_FONTS_PRIVATE(fonts);

  switch(prop_id)
  {
  case PROP_PREFERENCES:
    c4s_preferences_fonts_set_preferences(
      fonts,
      C4S_PREFERENCES(g_value_get_object(value))
    );

    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    break;
  }
}

static void
c4s_preferences_fonts_get_property(GObject* object,
                                   guint prop_id,
                                   GValue* value,
                                   GParamSpec* pspec)
{
  C4sPreferencesFonts* fonts;
  C4sPreferencesFontsPrivate* priv;

  g_return_if_fail(C4S_IS_PREFERENCES_FONTS(object));

  fonts = C4S_PREFERENCES_FONTS(object);
  priv = C4S_PREFERENCES_FONTS_PRIVATE(fonts);

  switch(prop_id)
  {
  case PROP_PREFERENCES:
    g_value_set_object(value, priv->preferences);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    break;
  }
}

static void
c4s_preferences_fonts_class_init(C4sPreferencesFontsClass* klass)
{
  GObjectClass* object_class = G_OBJECT_CLASS(klass);
  parent_class = g_type_class_peek_parent(klass);
  
  g_type_class_add_private(klass, sizeof(C4sPreferencesFontsPrivate));

  object_class->finalize = c4s_preferences_fonts_finalize;
  object_class->set_property = c4s_preferences_fonts_set_property;
  object_class->get_property = c4s_preferences_fonts_get_property;

  g_object_class_install_property(
    object_class,
    PROP_PREFERENCES,
    g_param_spec_object(
      "preferences",
      "Preferences",
      "Preferences to operate on",
      C4S_TYPE_PREFERENCES,
      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
    )
  );
}

GType
c4s_preferences_fonts_get_type(void)
{
  static GType our_type = 0;

  if(our_type == 0)
  {
    static const GTypeInfo our_info = {
      sizeof(C4sPreferencesFontsClass),
      (GBaseInitFunc)NULL,
      (GBaseFinalizeFunc)NULL,
      (GClassInitFunc)c4s_preferences_fonts_class_init,
      NULL,
      NULL,
      sizeof(C4sPreferencesFonts),
      0,
      (GInstanceInitFunc)c4s_preferences_fonts_init,
      NULL
    };

    our_type = g_type_register_static(
      C4S_TYPE_PREFERENCES_PAGE,
      "C4sPreferencesFonts",
      &our_info,
      0
    );
  }

  return our_type;
}

C4sPreferencesFonts*
c4s_preferences_fonts_new(C4sPreferences* preferences)
{
  GObject* object;

  object = g_object_new(
    C4S_TYPE_PREFERENCES_FONTS,
    "preferences", preferences,
    NULL
  );

  return C4S_PREFERENCES_FONTS(object);
}
