/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_REMOTE_COMM_H__
#define __C4S_REMOTE_COMM_H__

#include <gtk/gtkwindow.h>

#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_REMOTE_COMM              (c4s_remote_comm_get_type())
#define C4S_REMOTE_COMM(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_REMOTE_COMM, C4sRemoteComm))
#define C4S_REMOTE_COMM_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_REMOTE_COMM, C4sRemoteCommClass))
#define C4S_IS_REMOTE_COMM(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_REMOTE_COMM))
#define C4S_IS_REMOTE_COMM_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_REMOTE_COMM))
#define C4S_REMOTE_COMM_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_REMOTE_COMM, C4sRemoteCommClass))

typedef struct _C4sCommBackend C4sCommBackend;

typedef void(*C4sCommFileFunc)(const gchar*, gpointer);

typedef struct _C4sCommBackendFuncs C4sCommBackendFuncs;
struct _C4sCommBackendFuncs {
  C4sCommBackend*(*init)(C4sCommFileFunc, gpointer);
  void(*deinit)(C4sCommBackend*);

  gboolean(*open)(C4sCommBackend*, GtkWindow*);
  gboolean(*connect)(C4sCommBackend*);
  
  void(*send)(C4sCommBackend*, int, gchar* const *);
};

typedef struct _C4sRemoteCommClass C4sRemoteCommClass;
typedef struct _C4sRemoteComm C4sRemoteComm;

struct _C4sRemoteCommClass
{
  GObjectClass parent_class;
  
  void(*file_received)(C4sRemoteComm*, const gchar*);
};

GType
c4s_remote_comm_get_type(void) G_GNUC_CONST;

C4sRemoteComm*
c4s_remote_comm_new(const C4sCommBackendFuncs* backend);

gboolean
c4s_remote_comm_connect(C4sRemoteComm* comm);

gboolean
c4s_remote_comm_open(C4sRemoteComm* comm,
                     GtkWindow* window);

void
c4s_remote_comm_send(C4sRemoteComm* comm,
                     int argc,
                     gchar* const * argv);

G_END_DECLS

#endif /* __C4S_REMOTE_COMM_H__ */
